/* server.c */
/* Copyright (C) 1995 Eric Young (eay@mincom.oz.au).
 * All rights reserved.
 * Copyright remains Eric Young's, and as such any Copyright notices in
 * the code are not to be removed.
 * See the COPYRIGHT file in the SSLeay distribution for more details.
 */

#include <stdio.h>
#include <netdb.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/param.h>
#include <sys/errno.h>
#include <netinet/in.h>
#include "net.h"
#include "X509.h"
#include "ssl.h"

static int init_server();
int do_accept();
void do_server();

extern int errno;
void body();

#define BUFSIZZ	100*1024
static int accept_socket=-1;

#define TEST_CERT	"test.PEM"

main(argc,argv)
int argc;
char *argv[];
	{
	int port=PORT;

        if (argc>1)
            port=atoi(argv[1]);

	SSL_debug("server.log");
	if (!X509_set_default_verify_paths()) exit(1);
	do_server(SERVICE,port,NULL);
	return(0);
	}

void body(hostname,s)
char *hostname;
int s;
	{
	unsigned char buf[BUFSIZZ];
	fd_set readfds;
	int width;
	int i;
	SSL *con;

	fprintf(stderr,"CONNECTED\n");
	con=(SSL *)SSL_new();
	SSL_set_fd(con,s);

	if (SSL_use_RSAPrivateKey_file(con,TEST_CERT,X509_FILETYPE_PEM) <= 0)
		{
		fprintf(stderr,"unable to set public key\n");
		exit(1);
		}
	if (SSL_use_certificate_file(con,TEST_CERT,X509_FILETYPE_PEM) <= 0)
		{
		fprintf(stderr,"unable to set certificate\n");
		exit(1);
		}

	SSL_set_verify(con,SSL_VERIFY_NONE,NULL);

	if (!SSL_accept(con))
		{
		fprintf(stderr,"SSL error\n");
		fprintf(stderr,"function is:%s\n",
			SSL_error_func_string(SSL_errno));
		fprintf(stderr,"error is   :%s\n",
			SSL_error_string(SSL_errno));
		SSL_free(con);
		return;
		}
	printf("CIPHER is %s\n",SSL_get_cipher(con));

	width=s+1;
	for (;;)
		{
		FD_ZERO(&readfds);
		FD_SET(fileno(stdin),&readfds);
		FD_SET(s,&readfds);
		select(width,&readfds,NULL,NULL,NULL);
		if (FD_ISSET(fileno(stdin),&readfds))
			{
			i=read(fileno(stdin),buf,BUFSIZZ);
			if ((i <= 0) || (buf[0] == 'Q'))
				{
				fprintf(stderr,"DONE\n");
				close(s);
				shutdown(accept_socket,2);
				exit(0);
				}
			SSL_write(con,buf,i);
			}
		if (FD_ISSET(s,&readfds))
			{
			i=SSL_read(con,buf,BUFSIZZ);
			if (i<=0)
				{
				fprintf(stderr,"DONE\n");
				break;
				}
			write(fileno(stdout),buf,i);
			}
		}
	SSL_free(con);
	}

void close_accept_socket()
	{
	if (accept_socket >= 0)
		shutdown(accept_socket,2);
	}

void do_server(server,port,ret)
char *server;
int port;
int *ret;
	{
	int sock;
	char *name;

	if (!init_server(&accept_socket,server,port)) exit(1);

	if (ret != NULL)
		{
		*ret=accept_socket;
		return;
		}
	for (;;)
		{
		fprintf(stderr,"ACCEPT\n");
		if (do_accept(accept_socket,&sock,&name) == 0)
			exit(1);
		body(name,sock);
		free(name);
		close(sock);
		}
	}

static int init_server(sock,service,port)
int *sock;
char *service;
int port;
	{
	struct servent ser;
	struct servent *serv;
	struct sockaddr_in server;
	int s,i;
	struct linger ling;

	if ((serv=getservbyname(service,PROTO)) == NULL)
		{
		serv=&ser;
		ser.s_port=port;
		}

	server.sin_family=AF_INET;
	server.sin_port=serv->s_port;
	server.sin_addr.s_addr=INADDR_ANY;

	s=socket(AF_INET,SOCK_STREAM,0);
	if (s == -1)
		return(0);
	if (bind(s,(struct sockaddr *)&server,sizeof(server)) == -1)
		{
		perror("bind");
		return(0);
		}
	if (listen(s,5) == -1)
			return(0);
	ling.l_onoff=1;
	ling.l_linger=0;
	i=setsockopt(s,SOL_SOCKET,SO_LINGER,(char *)&ling,sizeof(ling));
	if (i < 0) { perror("linger"); exit(0); }

	i=0;
	*sock=s;
	return(1);
	}

int do_accept(acc_sock,sock,host)
int acc_sock;
int *sock;
char **host;
	{
	int ret,i;
	struct hostent *h1,*h2;
	static struct sockaddr_in from;
	int len;
	struct linger ling;

redoit:
	memset((char *)&from,0,sizeof(from));
	len=sizeof(from);
	ret=accept(acc_sock,(struct sockaddr *)&from,&len);
	if (ret == -1)
		{
		if (errno == EINTR)
			{
			/*check_timeout();/**/
			goto redoit;
			}
		perror("accept");
		return(0);
		}
	
	ling.l_onoff=1;
	ling.l_linger=0;
	i=setsockopt(ret,SOL_SOCKET,SO_LINGER,(char *)&ling,sizeof(ling));
	if (i < 0) { perror("linger"); exit(0); }
	i=0;
	i=setsockopt(ret,SOL_SOCKET,SO_KEEPALIVE,(char *)&i,sizeof(i));
	if (i < 0) { perror("keepalive"); exit(0); }

	h1=gethostbyaddr((char *)&from.sin_addr.s_addr,
		sizeof(from.sin_addr.s_addr),AF_INET);
	if (h1 == NULL)
		{
		fprintf(stderr,"bad gethostbyaddr\n");
		return(0);
		}
	if ((*host=(char *)malloc(strlen(h1->h_name)+1)) == NULL)
		{
		perror("malloc");
		return(0);
		}
	strcpy(*host,h1->h_name);

	h2=gethostbyname(h1->h_name);
	i=0;
	if (h2->h_addrtype != AF_INET)
		{
		fprintf(stderr,"gethostbyname addr is not AF_INET\n");
		return(0);
		}
	while (h2->h_addr_list[i])
		{
		if (memcmp(h2->h_addr_list[i],(char *)&from.sin_addr.s_addr,
			sizeof(from.sin_addr.s_addr)) == 0)
			{
			*sock=ret;
			return(1);
			}
		}
	return(0);
	}
